"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SPBinding = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /**
                                                                                                                                                                                                                                                                   * Think of this like Redux's "connect". It exports some functions that make it really easy for SP
                                                                                                                                                                                                                                                                   * react components to bind themselves to SP API objects.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * If I were doing everything from scratch, I'd probably just have sp-client use Redux internally;
                                                                                                                                                                                                                                                                   * it already uses some really similar methods. That may still be in the cards. But hopefully
                                                                                                                                                                                                                                                                   * using these methods here means we won't have to change any of our components when that internal
                                                                                                                                                                                                                                                                   * refactor happens.
                                                                                                                                                                                                                                                                   */

exports.watch = watch;
exports.bindComponent = bindComponent;

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * This maintains all of our watcher singletons. They're hashed by a simple string representation
 * of their arguments.
 */
var watcherSingletons = {};

/**
 * Helper function to tersely express monitoring changes of a Streamplace API object. This
 * function will always return the same object for the same incoming parameters. I think the word
 * for that is "memoized" but it's been a while.
 *
 * @param  {string} resource Resource to watch
 * @param  {Object} filter   Data filters to apply to the watch
 * @param  {Object} options
 */
function watch(resource, filter) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  // TODO: This string representation got us started, but this function is called a LOT and so we
  // should try and make it faster than this. Unless this string thing is truly the fastest way?
  var key = resource + "-" + JSON.stringify(filter) + "-" + JSON.stringify(options);
  if (!watcherSingletons[key]) {
    watcherSingletons[key] = { key: key, resource: resource, filter: filter, options: options };
  }
  return watcherSingletons[key];
}

/**
 * Common special case of watch() where we know there only exists one of the thing.
 * @param  {string} resource Resource to watch
 * @param  {Object} filter   Data filters to apply to the watch
 * @param  {Object} options  {one: true}
 * @return {Object}
 */
watch.one = function (resource, filter) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  return watch(resource, filter, _extends({}, options, { one: true }));
};

/**
 * This is the superclass that wraps all components that want to be bound to Streamplace API
 * objects.
 */

var SPBinding = exports.SPBinding = function (_Component) {
  _inherits(SPBinding, _Component);

  function SPBinding(props, _ref) {
    var SP = _ref.SP;

    _classCallCheck(this, SPBinding);

    var _this = _possibleConstructorReturn(this, (SPBinding.__proto__ || Object.getPrototypeOf(SPBinding)).call(this));

    _this.previousWatchers = {};
    _this.handles = {};
    _this.state = { SP: SP };
    return _this;
  }

  _createClass(SPBinding, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      this.resubscribe(this.props);
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(newProps) {
      this.resubscribe(newProps);
    }

    /**
     * Every time any value changes, we need to make sure that didn't cause it to change our
     * watchers. This function makes that check. This code path needs to be as fast as possible, so
     * be kinda careful.
     *
     * "But Eli, this isn't really all that optimized right now!"
     *
     * yeah, yeah, do as I say, not as I do, etc
     */

  }, {
    key: "resubscribe",
    value: function resubscribe() {
      var _this2 = this;

      var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props;
      var state = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.state;

      var newWatch = this.constructor.BoundComponent.subscribe(_extends({}, props, state));
      var oldWatch = this.previousWatchers;

      // Collect all the watch keys, old and new.
      var allKeys = [].concat(_toConsumableArray(new Set([].concat(_toConsumableArray(Object.keys(newWatch)), _toConsumableArray(Object.keys(this.previousWatchers))))));
      var changedKeys = allKeys.filter(function (key) {
        return newWatch[key] !== oldWatch[key];
      });

      if (changedKeys.length === 0) {
        return;
      }

      var SP = this.context.SP;

      if (!SP) {
        throw new Error("Missing SP in context somehow. This shouldn't happen.");
      }

      changedKeys.forEach(function (key) {
        // If it existed before, unsubscribe from that changefeed.
        if (oldWatch[key]) {
          _this2.handles[key].stop();
          delete _this2.handles[key];
          _this2.setState(_defineProperty({}, key, null));
        }

        // If it exists now, subscribe to a new changefeed.
        if (newWatch[key]) {
          var _newWatch$key = newWatch[key],
              resource = _newWatch$key.resource,
              filter = _newWatch$key.filter,
              options = _newWatch$key.options;

          if (!SP[resource]) {
            throw new Error("Tried to watch " + resource + " but I've never heard of that resource");
          }
          _this2.handles[key] = SP[resource].watch(filter).on("data", function (data) {
            if (options.one) {
              data = data[0];
            }
            // setState isn't sync, but we need to call resubscribe right away with the new data, so
            // here we are.
            var mod = _defineProperty({}, key, data);
            _this2.setState(mod);
            // This line right here is why this code needs to be so dang fast
            _this2.resubscribe(_this2.props, _extends({}, _this2.state, mod));
          }).catch(function (err) {
            SP.error("Error setting up watch", err);
          });
        }
      });

      this.previousWatchers = newWatch;
    }

    /**
     * We're leaving! Don't let the unsubscribe calls hit you on the way out!
     */

  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var _this3 = this;

      Object.keys(this.previousWatchers).forEach(function (key) {
        _this3.handles[key].stop();
        delete _this3.handles[key];
      });
    }

    /**
     * Render our bound friend. Pass them our props, plus our entire state.
     */

  }, {
    key: "render",
    value: function render() {
      var BoundComponent = this.constructor.BoundComponent;

      var props = _extends({}, this.props, this.state);
      return _react2.default.createElement(BoundComponent, props);
    }
  }]);

  return SPBinding;
}(_react.Component);

SPBinding.contextTypes = {
  SP: _propTypes2.default.object.isRequired
};


var noop = function noop() {
  return {};
};

function bindComponent(BoundComponent) {
  var _class, _temp;

  if (!BoundComponent.subscribe) {
    BoundComponent.subscribe = noop;
  }
  return _temp = _class = function (_SPBinding) {
    _inherits(Binding, _SPBinding);

    function Binding() {
      _classCallCheck(this, Binding);

      return _possibleConstructorReturn(this, (Binding.__proto__ || Object.getPrototypeOf(Binding)).apply(this, arguments));
    }

    return Binding;
  }(SPBinding), _class.BoundComponent = BoundComponent, _temp;
}
//# sourceMappingURL=sp-binding.js.map